/*
 * Copyright (C) 2012 Chris Burdess <dog@gnu.org>
 *
 * Gnome Shell Biff is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Gnome Shell Biff is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

const St = imports.gi.St;
const Main = imports.ui.main;
const GConf = imports.gi.GConf;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
//const Mainloop = imports.mainloop;
const Utils = imports.misc.util;
const GnomeKeyring = imports.gi.GnomeKeyring;
const EDataServer = imports.gi.EDataServer;
const Extension = imports.ui.extensionSystem.extensions["biff@bluezoo.org"];
const Imap = Extension.imap;
//const Logger = imports.logger.logger;

let check_interval, enabled;//, check_mail_pid;
let button, extension_path, icon_read, icon_unread, label;
let queue = null;
let checking = false, source, connections;

function ImapURL(url) {
    this.init(url);
}

ImapURL.prototype = {

init : function(url) {
    this.port = 143;
    let si = url.indexOf(":");
    this.scheme = url.substring(0, si);
    if (url.indexOf("security-method=ssl-on-alternate-port") != -1) {
        this.scheme = "imaps";
    }
    if (this.scheme === "imaps") {
        this.port = 993;
    }
    si += 3;
    let ei = url.indexOf("/", si);
    this.mailbox = url.substring(ei + 1);
    let ui = url.lastIndexOf("@", ei);
    this.host = url.substring(ui + 1, ei);
    let ci = this.host.lastIndexOf(":");
    if (ci != -1) {
        this.port = parseInt(this.host.substring(ci + 1));
        this.host = this.host.substring(0, ci);
    }
    this.username = url.substring(si, ui);
    this.password = null;
    ci = this.username.lastIndexOf(":");
    if (ci != -1) {
        this.password = GLib.uri_unescape_string(this.username.substring(ci + 1), "");
        this.username = this.username.substring(0, ci);
    }
    this.username = GLib.uri_unescape_string(this.username, "");
    ci = this.mailbox.indexOf(";");
    if (ci != -1) {
        this.mailbox = this.mailbox.substring(0, ci);
    }
    this.mailbox = GLib.uri_unescape_string(this.mailbox, "");
    if (this.mailbox.length == 0) {
        this.mailbox = "INBOX";
    }
}

};

function open_mua() {
    check_mail();
    try {
        let appinfo = Gio.app_info_get_default_for_type("x-scheme-handler/mailto", false);
        let success = appinfo.launch([], null);
        if (!success) {
            global.log("Error launching default MUA");
        }
    } catch (err) {
        global.log("Can't start default MUA: "+err.message);
    }
}

function check_mail() {
    try {
        if (!checking) {
            checking = true;
            connections = 0;
            //global.log("check mail");
            if (queue == null) {
                queue = get_mail_accounts();
            }
            for (let i = 0; i < queue.length; i++) {
                check_account(queue[i]);
            }
        }
    } catch (err) {
        global.log("check_mail: "+err.message);
    }
    return true; // continue this timer
}

function done_callback() {
    connections++;
    if (connections == queue.length) {
        checking = false;
        //global.log("check complete");
    }
}

function get_mail_accounts() {
    try {
        let ret = new Array();
        let gconfclient = GConf.Client.get_default();
        let list = gconfclient.get("/apps/evolution/mail/accounts").get_list(GConf.VALUE_STRING);
        for (let i = 0; i < list.length; i++) {
            let xml = list[i].get_string();
            // TODO reinstate this when EDataServer's e_account_get_string
            // works
            //let account = EDataServer.Account.new_from_xml(xml);
            //let url = account.get_string(6); // E_ACCOUNT_SOURCE_URL
            let si = xml.indexOf("<url>");
            let ei = xml.indexOf("</url>", si);
            let url = new ImapURL(xml.substring(si + 5, ei));
            if (url.scheme.indexOf("imap") == 0) {
                ret.push(url);
            }
        }
        return ret;
    } catch (err) {
        global.log("Error retrieving mail accounts: "+err.message);
    }
}

function check_account(url) {
    if (url.password == null) {
        url.password = get_password("imap", url.host, url.username);
        if (url.password == null) {
            url.password = get_password("imapx", url.host, url.username);
        }
    }
    if (url.password == null) {
        // TODO ask for password if none found and store it
        global.log("Can't retrieve password for "+url.username+" on "+url.host);
        return;
    }
    //global.log("checking "+url.host);
    let imap = new Imap.IMAPConnection(url.host, url.port, url.username, url.password, (url.scheme === "imaps"), url.mailbox, update_unread_count, done_callback);
    imap.connect();
}

function get_password(scheme, host, username) {
    let password = null;
    try {
        let type = 1; /* GNOME_KEYRING_ITEM_NETWORK_PASSWORD */
        let attributes = GnomeKeyring.attribute_list_new();
        GnomeKeyring.attribute_list_append_string(attributes, "protocol", scheme);
        GnomeKeyring.attribute_list_append_string(attributes, "server", host);
        GnomeKeyring.attribute_list_append_string(attributes, "user", username);
        let [result, found] = GnomeKeyring.find_items_sync(type, attributes);
        if (result == 0 /* GNOME_KEYRING_RESULT_OK */) {
            let f = found[0];
            password = f.secret;
        }
    } catch (err) {
        global.log("get_password: "+err.message);
    }
    return password;
}

function update_unread_count(account, value) {
    //global.log("update_unread_count "+account+"="+value);
    if (value != null) {
        try {
            let total = value;
            let gconfclient = GConf.Client.get_default();
            // GConfClient.set_list doesn't work, use string instead
            let countsval = gconfclient.get("/apps/gnome-biff/counts-string");
            let counts = null;
            let acc = account + "=" + value;
            if (countsval != null) {
                counts = countsval.get_string();
            }
            if (counts == null) {
                counts = "";
            }
            let tokens = counts.split(",");
            for (let i = 0; i < tokens.length; i++) {
                let line = tokens[i].trim();
                if (line.length > 0) {
                    let ei = line.lastIndexOf("=");
                    let head = line.substring(0, ei);
                    let tail = line.substring(ei + 1);
                    if (head === account) {
                    } else {
                        total = total + parseInt(tail);
                        acc = acc + "," + line;
                    }
                }
            }
            gconfclient.set_string("/apps/gnome-biff/counts-string", acc);
            set_message_count(total);
        } catch (err) {
            global.log("Error updating new message count list: "+err.message);
        }
    }
}

function init(meta) {
    extension_path = meta.path;
    _init();
}

function enable() {
    enabled = true;
    Main.panel._rightBox.insert_actor(button, 0);
    check_interval = 20;
    let gconfclient = GConf.Client.get_default();
    let cci = gconfclient.get("/apps/gnome-biff/check-interval");
    if (cci != null) {
        check_interval = cci.get_int();
        if (check_interval < 10) {
            check_interval = 10; // don't invoke check_mail too frequently
        }
    }
    //check_mail_pid = GLib.timeout_add_seconds(0, check_interval, check_mail);
    source = GLib.timeout_source_new_seconds(check_interval);
    source.set_name("biff");
    source.set_priority(20);
    source.set_can_recurse(false);
    source.set_callback(check_mail, null, null);
    source.ref();
    source.attach(null);
    check_mail();
}

function disable() {
    enabled = false;
    Main.panel._rightBox.remove_actor(button);
    /*if (check_mail_pid != null) {
        Mainloop.source_remove(check_mail_pid);
        check_mail_pid = null;
    }*/
    if (source != null) {
        source.unref();
        source.destroy();
        source = null;
    }
}

// TODO move this to a separate prototype

function _init() {
    button = new St.Bin({ style_class: 'panel-button', reactive: true, can_focus: true, x_fill:true, y_fill: false, track_hover:true });
    icon_read = new St.Icon({ icon_name: 'mail-read-symbolic', icon_size: 16, icon_type: St.IconType.SYMBOLIC, style_class: 'system_status_icon' });
    icon_unread = new St.Icon({ icon_name: 'mail-unread-symbolic', icon_size: 16, icon_type: St.IconType.SYMBOLIC, style_class: 'system_status_icon' });
    icon_unread.hide();
    label = new St.Label({ style_class: 'button_text', text: '...' });
    let box = new St.BoxLayout();
    box.insert_actor(this.icon_read, 1);
    box.insert_actor(this.icon_unread, 1);
    box.insert_actor(this.label, 2);
    button.set_child(box);
    button.connect('button-press-event', open_mua);
}

function set_message_count(count) {
    if (count > 0) {
        label.text = count.toString();
        icon_read.hide();
        icon_unread.show();
        label.show();
    } else {
        icon_unread.hide();
        icon_read.show();
        label.hide();
    }
}

